package edgebundles.visualisation.tree;

import java.awt.Graphics;

import edgebundles.tree.VisualTree;

/**
 * <p>Title: Tree Visualiser</p>
 *
 * <p>Description: Abstract Class and a factory for all tree visualizers</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * <p>Company: </p>
 *
 * @author Bilal Alsallakh
 * @version 1.0
 */
public abstract class TreeVisualiser {

    /**
     * visualise
     *
     * @param tree VisualTree: The tree to visualise
     * @param g Graphics: port to a canvas with drawing primitives
     * @param width: The width (in pixels) of the drawing area
     * @param height: The height (in pixels) of the drawing area
     *
     * The visualisation is performed for all layouts in two major steps,
     * first the nodes are assigned relative coordinates, then the actual
     * placement and size for each node is calculated, based on the available
     * drawing space. The RandomLayout is an exception since it assigns
     * random coordinates for the nodes inside the available space.
     */

    public abstract void visualise(VisualTree tree, Graphics g,
                                   int width, int height);

    // list of tree visualisers
    private static TreeVisualiser[] treeVis = {
                               new RadialLayoutVisualiser(),
                               new InvertedRadialLayoutVisualiser(),
                               new RootedLayoutVisualiser(),
                               new InvertedRootedLayoutVisualiser(),
                               new BalloonLayoutVisualiser(),
                               new TreeMapLayoutVisualiser(),
                               new RandomTreeVisualiser()
    };

    /**
     * getVisualisersCount
     *
     * returns the number of available visualisers
     */

    public static int getVisualisersCount() {
        return treeVis.length;
    }

    /**
     * getVisualiser
     *
     * returns the ith visualiser in the visualisers list.
     * @param i: the index of the visualiser
     */

    public static TreeVisualiser getVisualiser(int i) {
        return treeVis[i];
    }
}
